/**************************************************************************//**
 * @file     main.c
 * @version  V2.03
 * $Revision: 2 $
 * $Date: 13/06/28 10:14a $ 
 * @brief    M051 Series General Purpose I/O Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "M051Series.h"
#include <DMX512.h>


#define DMX512_CLK __IRC22M
#define DMX512_BR	250000
#define DMX512_UART UART0
uint8_t u8DMX512[DMX512_size] = {0,1,2,4,8,16,32,64,128,1,2,4,8,16,32,64,128,1,2,4,8,16,32,64,128};
uint32_t u32DMXcount;
uint8_t u8DMX512data;
uint8_t u8DMX512flag = 1;

void DMX512_Init(void)
{
/*---------------------------------------------------------------------------------------------------------*/
/* Init UART                                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
    /* Enable UART clock */        
    SYSCLK->APBCLK |= SYSCLK_APBCLK_UART0_EN_Msk;	
    /* Set P3 multi-function pins for UART0 RXD and UART0 TXD */
    SYS->P3_MFP = SYS_MFP_P30_RXD0 | SYS_MFP_P31_TXD0;	
    /* Set P0 multi-function pins for UART0 RTS */
    SYS->P0_MFP = SYS_MFP_P03_RTS0;		
    /* Reset UART */
    SYS->IPRSTC2 |=  SYS_IPRSTC2_UART0_RST_Msk;
    SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART0_RST_Msk;
    NVIC_EnableIRQ(UART0_IRQn);	
	
    /* Configure UART0 and set UART0 Baudrate */
    DMX512_UART->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(DMX512_CLK, DMX512_BR);
    _UART_SET_DATA_FORMAT(DMX512_UART, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_2);
    /* Set RX Trigger Level = 1 */
    _UART_SET_RX_TRG_LEV(DMX512_UART,UART_FCR_RFITL_1BYTE);		
		DMX512_UART->MCR &= ~UART_MCR_RTS_Msk;//Rx		
    /* Enable RDA\RLS Interrupt  */
    _UART_ENABLE_INT(DMX512_UART, (UART_IER_RDA_IEN_Msk | UART_IER_RLS_IEN_Msk));
}

/*---------------------------------------------------------------------------------------------------------*/
/* ISR to handle UART Chann0el 0 interrupt event                                                            */
/*---------------------------------------------------------------------------------------------------------*/
void UART0_IRQHandler(void)
{
    if((DMX512_UART->ISR & UART_ISR_RLS_INT_Msk))           /* RLS INT */  //For RS485 Detect Address
    {   
        if((DMX512_UART->FSR & (UART_FSR_BIF_Msk|UART_FSR_FEF_Msk))==(UART_FSR_BIF_Msk|UART_FSR_FEF_Msk))  /* ADD_IF, RS485 mode */
        {//Receive break signal   
					u32DMXcount = 0;
					P43 = ~P43;
				}else
				{//Receive undefined signal
					u32DMXcount = DMX512_size+1;
					u8DMX512flag = 0;
				}
				_UART_CLEAR_RLS_INT_FLAG(DMX512_UART);        /* clear RLS_IF flag */				
				u8DMX512data = DMX512_UART->DATA;//Read Rx FIFO				
    }
    if( DMX512_UART->ISR & UART_ISR_RDA_INT_Msk ) /* Rx Ready */   
    {
        /* Handle received data */
				if(u32DMXcount<DMX512_size)
					u8DMX512[u32DMXcount++] = DMX512_UART->DATA;
				else
					u8DMX512data = DMX512_UART->DATA;
				
				if(u32DMXcount == DMX512_size)
				{
					u32DMXcount = DMX512_size+1;
					u8DMX512flag = 1;
				}
    }
}


void DMX512_SendData(uint8_t* p8Data)
{
		uint32_t i;
    /* Configure UART0 and set UART0 Baudrate */
		_UART_WAIT_TX_EMPTY(DMX512_UART);
		DMX512_UART->MCR |= UART_MCR_RTS_Msk;//TX	
		_UART_SET_DATA_FORMAT(DMX512_UART, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1);
    DMX512_UART->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(DMX512_CLK, (DMX512_BR/3));	
		DMX512_UART->DATA = 0x00;
		_UART_WAIT_TX_EMPTY(DMX512_UART);
		_UART_SET_DATA_FORMAT(DMX512_UART, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_2);
		DMX512_UART->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(DMX512_CLK, DMX512_BR);	
		for(i=0;i<DMX512_size;i++)
		{
			while(_UART_IS_TX_FULL(DMX512_UART));
			DMX512_UART->DATA = p8Data[i];
		}
		_UART_WAIT_TX_EMPTY(DMX512_UART);
		DMX512_UART->MCR &= ~UART_MCR_RTS_Msk;//Rx		
}

