/**************************************************************************//**
 * @file     main.c
 * @version  V2.03
 * $Revision: 2 $
 * $Date: 13/06/28 10:14a $ 
 * @brief    M051 Series General Purpose I/O Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "M051Series.h"

uint8_t u8PNewPIN = 0;
uint8_t u8POldPIN = 0;
uint8_t u8PWaitStatus = 0;
uint8_t u8PProcStatus = 0;//Set default value to detect key when power on

extern uint8_t u8Mode;
extern uint8_t u8Power;
extern uint8_t u8Modeflag;
extern uint8_t u8Powerflag;

void Key_Power(void)
{
	u8Powerflag = 1;
	u8Power=~u8Power;
}
void Key_Mode(void)
{
	if(u8Power)
	{
		u8Mode++;
		if(u8Mode>=3)
			u8Mode = 0;
		u8Modeflag = 1;
	}
}

void Key_Init(void)
{
	_GPIO_SET_PIN_MODE(P4, 0,GPIO_PMD_QUASI);	
	_GPIO_SET_PIN_MODE(P4, 2,GPIO_PMD_QUASI);	
	SYS_SysTickDelay(20000);
	u8POldPIN = P4->PIN & (GPIO_ISRC_ISRC0 | GPIO_ISRC_ISRC2);
	GPIO_EnableInt(P4, 0, GPIO_INT_BOTH_EDGE);
	GPIO_EnableInt(P4, 2, GPIO_INT_BOTH_EDGE);
	NVIC_EnableIRQ(GPIO_P2P3P4_IRQn);	
	
	_TIMER_RESET(TIMER1);

	/* Enable peripheral clock */        
	SYSCLK->APBCLK |= SYSCLK_APBCLK_TMR1_EN_Msk;

	/* Peripheral clock source */
	SYSCLK->CLKSEL1 |= SYSCLK_CLKSEL1_TMR1_IRC22M;

	TIMER1->TCMPR = (__IRC22M/50);       

	/* Start TIMER0 counting */
	//TIMER1->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_ONESHOT | TIMER_TCSR_PRESCALE(1);
	TIMER1->TCSR = TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_ONESHOT | TIMER_TCSR_PRESCALE(1);
	
	/* Enable TIMER0 NVIC */
	NVIC_EnableIRQ(TMR1_IRQn);	
}

void GPIOP2P3P4_IRQHandler(void)
{
	uint32_t u8PStatus;
	uint8_t u8PAction;

	/* Keep the interrupt source */
	u8PStatus = P4->ISRC;
	
  u8PAction = u8PStatus &(GPIO_ISRC_ISRC0 | GPIO_ISRC_ISRC2);/*Check input with debounce*/

	
	if(u8PProcStatus)
	{
	    u8PWaitStatus |= u8PAction;

	}else
	{
		if(u8PAction)
		{
		    u8PProcStatus = u8PAction;
				TIMER1->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_ONESHOT | TIMER_TCSR_PRESCALE(1);
		}
	}	
  /* Clear the interrupt */
  P4->ISRC = u8PStatus;
}
void TMR1_IRQHandler(void)
{
	uint8_t u8PAction;
	/* Clear TIMER0 Time-out Interrupt Flag */
	_TIMER_CLEAR_CMP_INT_FLAG(TIMER1);  
	
	u8PNewPIN = P4->PIN;
	u8PAction = (u8PNewPIN^u8POldPIN)&u8PProcStatus;//Check pin active
	u8POldPIN = (u8POldPIN&~u8PProcStatus)|(u8PNewPIN&u8PProcStatus);//Store pin state
	
	if(u8PAction & GPIO_ISRC_ISRC0)	
	{
		if(u8POldPIN & GPIO_ISRC_ISRC0)
		{
		}else
		{
			Key_Power();
		}
	}
	if(u8PAction & GPIO_ISRC_ISRC2)	
	{
		if(u8POldPIN & GPIO_ISRC_ISRC2)
		{
		}else
		{
			Key_Mode();
		}
	}
	__disable_irq();
	if(u8PWaitStatus)
	{
		u8PProcStatus = u8PWaitStatus;
		u8PWaitStatus = 0;
		TIMER1->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_ONESHOT | TIMER_TCSR_PRESCALE(1);	
	}else
	{
		u8PProcStatus = 0;
	}
	__enable_irq();
}

