/****************************************************************************
 * @file     Smpl_DrvUART.c
 * @version  V1.00
 * $Revision: 6 $
 * $Date: 12/08/09 7:02p $ 
 * @brief    NUC200 Series UART Interface Controller Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "NUC200Series.h"

#define DMX512_BR 250000
#define DMX512_size 513
#define DMX512_CLK __XTAL


void UART0_Init()
{
    /* Select IP clock source */
    SYSCLK->CLKSEL1 = (SYSCLK->CLKSEL1 & ~(SYSCLK_CLKSEL1_UART_IRC22M)) |SYSCLK_CLKSEL1_UART_XTAL;    	

    /* Enable IP clock */        
    SYSCLK->APBCLK |= SYSCLK_APBCLK_UART0_EN_Msk;
    
		SYS->GPB_MFP = SYS_GPB_MFP_PB0_RXD0 | SYS_GPB_MFP_PB1_TXD0;
	/*---------------------------------------------------------------------------------------------------------*/
/* Init UART                                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
    UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(DMX512_CLK,DMX512_BR);
   
    _UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_2);
}

void DMX512_SendData(uint8_t* p8Data)
{
		uint32_t i;
    /* Configure UART0 and set UART0 Baudrate */
		_UART_WAIT_TX_EMPTY(UART0);
		_UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1);	
		UART0->MCR |= UART_MCR_RTS_Msk;//TX	

    UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(DMX512_CLK, (DMX512_BR/3));	
		UART0->DATA = 0x00;
		_UART_WAIT_TX_EMPTY(UART0);
		_UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_2);
		UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(DMX512_CLK, DMX512_BR);	
		for(i=0;i<DMX512_size;i++)
		{
			while(_UART_IS_TX_FULL(UART0));
			UART0->DATA = p8Data[i];
		}
		_UART_WAIT_TX_EMPTY(UART0);
		UART0->MCR &= ~UART_MCR_RTS_Msk;//Rx		
}


