/****************************************************************************
 * @file     Smpl_DrvUART.c
 * @version  V1.00
 * $Revision: 6 $
 * $Date: 12/08/09 7:02p $ 
 * @brief    NUC200 Series UART Interface Controller Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "NUC200Series.h"

#define PLLCON_SETTING      SYSCLK_PLLCON_50MHz_XTAL
#define PLL_CLOCK           50000000
__IO uint32_t g_u32AdcIntFlag;

/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Define functions prototype                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
void ADC_Init(void)
{
    /* Enable IP clock */        
    SYSCLK->APBCLK |= SYSCLK_APBCLK_ADC_EN_Msk ;	
		/* Set ADC divisor */
    _ADC_SET_CLK_DIV(7);
	
    /* Disable the GPA0 - GPA3 digital input path to avoid the leakage current. */
    PA->OFFD = GPIO_OFFD_ENABLE(7);
    /* Configure the GPA0 - GPA3 ADC analog input pins */
    SYS->GPA_MFP |= SYS_GPA_MFP_PA7_ADC7 ;

		/* Set the ADC operation mode as single, input mode as single-end and enable the ADC converter */
		ADC->ADCR = (ADC_ADCR_ADMD_SINGLE | ADC_ADCR_DIFFEN_SINGLE_END | ADC_ADCR_ADEN_CONVERTER_ENABLE);
		/* Enable analog input channel 2 */
		_ADC_SET_CHANNEL(1<<7);	
	
		/* clear the A/D interrupt flag for safe */
		ADC->ADSR = ADC_ADSR_ADF_Msk;
		
		/* Enable the ADC interrupt */
		_ADC_ENABLE_ADC_INT();             
		NVIC_EnableIRQ(ADC_IRQn); 		
}


uint16_t TriggerADC(void)
{
	/* Reset the ADC interrupt indicator and Start A/D conversion */
	g_u32AdcIntFlag = 0;
	_ADC_START_CONVERT();
	
	/* Wait ADC interrupt (g_u32AdcIntFlag will be set at IRQ_Handler Fuction)*/
	while(g_u32AdcIntFlag==0);	
	return _ADC_GET_CONVERSION_DATA(7);
}
/*---------------------------------------------------------------------------------------------------------*/
/* ADC interrupt handler                                                                                   */
/*---------------------------------------------------------------------------------------------------------*/
void ADC_IRQHandler(void)
{
    if(ADC->ADSR & ADC_ADSR_ADF_Msk)
    {
        g_u32AdcIntFlag = 1;

        ADC->ADSR = ADC_ADSR_ADF_Msk;      /* clear the A/D conversion flag */
    }
   
}




/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();
   /* Enable External XTAL (4~24 MHz) */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_XTL12M_EN_Msk;	
    /* Waiting for 12MHz clock ready */
    SYS_WaitingForClockReady( SYSCLK_CLKSTATUS_XTL12M_STB_Msk);		
	
    UART0_Init();
		ADC_Init();
		while(1){
// 			drawchar('N');
// 			drawchar('U');
// 			drawchar('V');
// 			drawchar('O');
// 			drawchar('T');
//			drawchar('N');
 			drawString("I Love You");
 			drawLogo("Nuvoton");
			while(1)
			{
				drawwave(TriggerADC());
			}
		}
/*---------------------------------------------------------------------------------------------------------*/
/* SAMPLE CODE                                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
}

//ADC: wave, output pwm
//USB VCOM: char
//up down Key: Mode
//left right key: speed or light
//


